/**
 * @file
 * Implements SearchStax tracking functionality.
 */

// General SearchStax tracking code.
var _msq = _msq || []; //declare object

(function () {
  var ms = document.createElement('script');
  ms.type = 'text/javascript';
  const jsVersion =
    typeof drupalSettings !== 'undefined' &&
    drupalSettings.searchstax &&
    drupalSettings.searchstax.js_version;
  switch (jsVersion) {
    case '2':
      ms.src = 'https://static.searchstax.com/js/ms2.js';
      break;

    case '3':
    default:
      ms.src =
        'https://static.searchstax.com/studio-js/v3/js/studio-analytics.js';
      window.analyticsBaseUrl =
        (typeof drupalSettings !== 'undefined' &&
          drupalSettings.searchstax &&
          drupalSettings.searchstax.analytics_url) ||
        'https://app.searchstax.com';
      break;
  }
  var s = document.getElementsByTagName('script')[0];
  s.parentNode.insertBefore(ms, s);
})();

// Drupal-specific tracking code.
(function ($) {

  'use strict';

  Drupal.searchstax = Drupal.searchstax || {};

  /**
   * Click handler for search result links.
   */
  Drupal.searchstax.trackSearchResults = function () {
    if (!drupalSettings
        || !drupalSettings.searchstax
        || !drupalSettings.searchstax.searches
        || typeof _msq === 'undefined') {
      return;
    }

    var base_data = drupalSettings.searchstax.tracking_base_data;
    if (!base_data.session) {
      // This is an anonymous user, without fixed session. Generate a new random
      // session key for just this page.
      // See https://attacomsian.com/blog/javascript-generate-random-string.

      // Declare all characters.
      var chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

      // Pick characters randomly.
      base_data.session = '';
      for (var i = 0; i < 32; i++) {
        base_data.session += chars.charAt(Math.floor(Math.random() * chars.length));
      }
    }
    $.each(drupalSettings.searchstax.searches, function () {
      // Don't track the same result set twice.
      if (this.tracked) {
        return;
      }
      this.tracked = true;
      var track = $.extend({}, base_data, this);
      _msq.push(['track', track]);
    });
  };

  /**
   * Click handler for search result links.
   */
  Drupal.searchstax.trackClick = function () {
    var linkElement = this;
    var $link = $(this);
    var position = $link.data('searchstax-position');
    var search_id = $link.data('searchstax-search');
    var searches = drupalSettings.searchstax.searches;
    if (!searches[search_id] || !searches[search_id].impressions) {
      return;
    }
    $.each(searches[search_id].impressions, function () {
      if (this.position !== position) {
        return;
      }

      var base_data = drupalSettings.searchstax.tracking_base_data;
      var track = $.extend({}, base_data, this, {
        query: searches[search_id].query,
        pageNo: searches[search_id].pageNo,
        shownHits: searches[search_id].shownHits,
        totalHits: searches[search_id].totalHits,
        pageUrl: linkElement.href,
      });
      _msq.push(['trackClick', track]);
      return false;
    });
  };

  /**
   * Enables tracking for clicks on all search results links.
   */
  Drupal.behaviors.searchstax = {
    attach: function (context, settings) {
      if (!settings
          || !settings.searchstax
          || !settings.searchstax.searches
          || typeof _msq === 'undefined') {
        return;
      }

      // First off, track all search results (that weren't tracked already).
      Drupal.searchstax.trackSearchResults();

      if (!settings.searchstax.results_urls) {
        return;
      }

      var result_urls = drupalSettings.searchstax.results_urls;
      var $search_results = $('[data-searchstax-results]', context);
      // For AJAX responses, "context" itself might be a search results listing.
      var $context = $(context);
      if ($context.data('searchstax-results')) {
        $search_results = $search_results.add($context);
      }
      $search_results.each(function () {
        var $results_area = $(this);
        var search_id = $results_area.data('searchstax-results');
        if (!result_urls[search_id]) {
          return;
        }
        $.each(result_urls[search_id], function () {
          $results_area.find('a[href="' + this.url + '"]')
            .data('searchstax-position', this.position)
            .data('searchstax-search', search_id)
            .click(Drupal.searchstax.trackClick);
        });
      });
    }
  };

})(jQuery);
