<?php

namespace Drupal\searchstax\EventSubscriber;

use Drupal\search_api_solr\Event\PreQueryEvent;
use Drupal\search_api_solr\Event\SearchApiSolrEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Listens to events dispatched by the Search API Solr module.
 */
class SearchSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $listeners = [];
    // Check if the event even exists, otherwise the alter hook implementation
    // will be used.
    if (class_exists(SearchApiSolrEvents::class)) {
      $listeners[SearchApiSolrEvents::PRE_QUERY][] = ['preQueryAlter'];
    }
    return $listeners;
  }

  /**
   * Reacts to the Search API Solr module's pre-query event.
   *
   * @param \Drupal\search_api_solr\Event\PreQueryEvent $event
   *   The pre-query event.
   *
   * @see _searchstax_search_alter_solr_query()
   */
  public function preQueryAlter(PreQueryEvent $event): void {
    _searchstax_search_alter_solr_query($event->getSolariumQuery(), $event->getSearchApiQuery());
  }

}
